
/***************************************************************************
 * nbase_memalloc.c -- A few simple functions related to memory            *
 * allocation.  Right now they are just safe_ versions of well known calls *
 * like malloc that quit if the allocation fails (so you don't have to     *
 * have a bunch of ugly checks in your code.  These were written by        *
 * fyodor@insecure.org .                                                   *
 *                                                                         *
 ***********************IMPORTANT NMAP LICENSE TERMS************************
 *                                                                         *
 * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
 * program is free software; you may redistribute and/or modify it under   *
 * the terms of the GNU General Public License as published by the Free    *
 * Software Foundation; Version 2.  This guarantees your right to use,     *
 * modify, and redistribute this software under certain conditions.  If    *
 * you wish to embed Nmap technology into proprietary software, we may be  *
 * willing to sell alternative licenses (contact sales@insecure.com).      *
 * Many security scanner vendors already license Nmap technology such as   *
 * our remote OS fingerprinting database and code.                         *
 *                                                                         *
 * Note that we consider aggregation/inclusion/integration of Nmap into an *
 * executable installer to constitute a derived work and thus subject to   *
 * the GPL restrictions.  We also consider certain programs that tightly   *
 * integrate with Nmap to constitute derivative works, even if they only   *
 * interface with Nmap by executing the Nmap binary and interpreting its   *
 * output rather than by direct linking.  If you are interested in         *
 * including Nmap with your proprietary software or appliance, please      *
 * contact us first to ensure proper licensing.                            *
 *                                                                         *
 * If you received these files with a written license agreement or         *
 * contract stating terms other than the (GPL) terms above, then that      *
 * alternative license agreement takes precedence over these comments.     *
 *                                                                         *
 * Source is provided to this software because we believe users have a     *
 * right to know exactly what a program is going to do before they run it. *
 * This also allows you to audit the software for security holes (none     *
 * have been found so far).                                                *
 *                                                                         *
 * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 * and add new features.  You are highly encouraged to send your changes   *
 * to fyodor@insecure.org for possible incorporation into the main         *
 * distribution.  By sending these changes to Fyodor or one the            *
 * Insecure.Org development mailing lists, it is assumed that you are      *
 * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
 * to reuse, modify, and relicense the code.  Nmap will always be          *
 * available Open Source, but this is important because the inability to   *
 * relicense code has caused devastating problems for other Free Software  *
 * projects (such as KDE and NASM).  We also occasionally relicense the    *
 * code to third parties as discussed above.  If you wish to specify       *
 * special license conditions of your contributions, just say so when you  *
 * send them.                                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details at                              *
 * http://www.gnu.org/copyleft/gpl.html .                                  *
 *                                                                         *
 ***************************************************************************/

/* $Id: nbase_memalloc.c,v 1.2 2003/09/06 09:10:55 fyodor Exp $ */

#include "nbase.h"
#include <stdio.h>

static void fatal(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));

static void fatal(char *fmt, ...) 
{
va_list ap;
va_start(ap, fmt);
fflush(stdout);
vfprintf(stderr, fmt, ap);
fprintf(stderr, "\nQUITTING!\n");
va_end(ap);
exit(1);
}

void *safe_malloc(int size)
{
  void *mymem;
  if (size < 0)
    fatal("Tried to malloc negative amount of memory!!!");
  mymem = malloc(size);
  if (mymem == NULL)
    fatal("Malloc Failed! Probably out of space.");
  //  printf("Called safe_malloc(%d) -- returning %lX\n", size, (unsigned long) mymem);
  return mymem;
}

void *safe_realloc(void *ptr, size_t size)
{
  void *mymem;
  if (size < 0)
    fatal("Tried to realloc negative amount of memory!!!");
  mymem = realloc(ptr, size);
  if (mymem == NULL)
    fatal("Realloc Failed! Probably out of space.");
  //  printf("Called safe_malloc(%d) -- returning %lX\n", size, (unsigned long) mymem);
  return mymem;
}

/* Zero-initializing version of safe_malloc */
void *safe_zalloc(int size)
{
  void *mymem;
  if (size < 0)
    fatal("Tried to malloc negative amount of memory!!!");
  mymem = calloc(1, size);
  if (mymem == NULL)
    fatal("Malloc Failed! Probably out of space.");
  //  printf("Called safe_zalloc(%d) -- returning %lX\n", size, (unsigned long) mymem);
  return mymem;
}
